<?php

namespace Drupal\svg_sprite\Service;

use Drupal\Core\Extension\ThemeExtensionList;
use Drupal\Core\Logger\LoggerChannelTrait;
use GuzzleHttp\Client;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Various helper functions for the svg_sprite module.
 *
 * @package Drupal\svg_sprite
 */
class SvgSpriteService {

  use LoggerChannelTrait;

  const SOURCE_THEME_PATTERN = '/theme:\/\/([^\/]+)\/(.*)/';

  /**
   * The theme manager.
   *
   * @var \Drupal\Core\Extension\ThemeExtensionList
   */
  protected ThemeExtensionList $extensionListTheme;

  /**
   * HTTP client service.
   *
   * @var \GuzzleHttp\Client
   */
  protected Client $httpClient;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * Constructor of the SVG Sprite service.
   *
   * @param \Drupal\Core\Extension\ThemeExtensionList $extension_list_theme
   *   The theme manager.
   * @param \GuzzleHttp\Client $httpClient
   *   The Drupal http client.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   */
  public function __construct(ThemeExtensionList $extension_list_theme, Client $httpClient, RequestStack $request_stack) {
    $this->extensionListTheme = $extension_list_theme;
    $this->httpClient = $httpClient;
    $this->request = $request_stack->getCurrentRequest();
  }

  /**
   * Fetch SVG data from the specified source.
   */
  public function fetchSvgDataFromSource(string $svg_source) {
    if (str_starts_with($svg_source, 'http')) {
      try {
        $response = $this->httpClient->get($svg_source);
        return $response->getBody()->getContents();
      }
      catch (\Throwable $e) {
        $this->getLogger('svg_sprite')->error($e->getMessage());
      }
    }
    else {
      if (str_starts_with($svg_source, 'theme://')) {
        if (preg_match(self::SOURCE_THEME_PATTERN, $svg_source, $matches)) {
          $theme_name = $matches[1];
          $asset_path = $matches[2];
          $theme_path = $this->extensionListTheme->getPath($theme_name);
          $svg_file = $theme_path . '/' . $asset_path;
        }
        else {
          return FALSE;
        }
      }
      else {
        $svg_file = trim($svg_source, '/');
      }
      return file_get_contents($svg_file);
    }
    return FALSE;
  }

  /**
   * Fetch SVG data from the specified source.
   */
  public function getSvgHref(string $svg_source) {
    if (str_starts_with($svg_source, 'http')) {
      return $svg_source;
    }
    else {
      if (str_starts_with($svg_source, 'theme://')) {
        if (preg_match(self::SOURCE_THEME_PATTERN, $svg_source, $matches)) {
          $theme_name = $matches[1];
          $asset_path = $matches[2];
          $theme_path = $this->extensionListTheme->getPath($theme_name);
          $svg_file = $theme_path . '/' . $asset_path;
        }
        else {
          return FALSE;
        }
      }
      else {
        $svg_file = trim($svg_source, '/');
      }
      $href = $this->request->getBasePath() . '/' . $svg_file;
      if (file_exists($svg_file)) {
        $href .= '?v=' . filemtime($svg_file);
      }
      return $href;
    }
  }

  /**
   * Extracts IDs and Aria labels from an SVG sprites file.
   */
  public function extractSpriteInfoFromSvgData(string $svg_data): array {
    $sprites = [];

    // Parse the SVG file as XML.
    $xml = simplexml_load_string($svg_data);
    $defs = $xml->defs;

    // If <defs> is found, extract the <symbol> tags.
    if ($defs) {
      // Find all <symbol> elements inside <defs>.
      $symbols = $defs->symbol;

      // Display the symbols.
      foreach ($symbols as $symbol) {
        // Get the id of the symbol.
        $symbolId = (string) $symbol['id'];
        $symbolAriaLabel = (string) $symbol['aria-label'];
        $sprites[$symbolId] = $symbolAriaLabel;
      }
    }

    return $sprites;
  }

}
